_PROJECT_ID_ENV_VAR = "MANTIK_PROJECT_ID"
_DATA_REPOSITORY_ID_ENV_VAR = "MANTIK_DATA_REPOSITORY_ID"
_EXPERIMENT_REPOSITORY_ID_ENV_VAR = "MANTIK_EXPERIMENT_REPOSITORY_ID"
_CODE_REPOSITORY_ID_ENV_VAR = "MANTIK_CODE_REPOSITORY_ID"
